@@ -1,4 +1,4 @@
-.CVAcrova <- function(dataarray,groups,test ,weighting=TRUE,tolinv=1e-10)
+.CVAcrova <- function(dataarray,groups,test ,weighting=TRUE,tolinv=1e-10,robust=c("classical", "mve", "mcd"),...)
 {   
     groups <- factor(groups)
     lev <- levels(groups)
@@ -16,17 +16,13 @@
     if (is.vector(N) || dim(N)[2] == 1)
         stop("data should contain at least 2 variable dimensions")
     
-    Gmeans <- matrix(0, ng, l)
-    rownames(Gmeans) <- lev
-    for (i in 1:ng) {
-        Gmeans[i, ] <- colMeans(N[groups==lev[i], ,drop=FALSE])
-    }
+    covWithin <- covW(N, groups,robust=robust,...)
+    Gmeans <- attributes(covWithin)$means
     if (weighting) {
         Grandm <- colSums(Gmeans*gsizes)/n ## calculate weighted Grandmean (thanks to Anne-Beatrice Dufour for the bug-fix)
     } else {
         Grandm <- colMeans(Gmeans)
     }
-    Tmatrix <- N
     N <- sweep(N, 2, Grandm) #center data according to Grandmean
     resGmeans <- sweep(Gmeans, 2, Grandm)
     
@@ -37,32 +33,29 @@
     } else 
         X <- sqrt(n/ng) * resGmeans
     
-    covW <- covW(N, groups)
-    eigW <- eigen(covW*(n - ng))
-    eigcoW <- eigen(covW)
-    U <- eigW$vectors
-    E <- eigW$values
+    eigcoW <- eigen(covWithin); ## eigen decomp of between group covariance Matrix
+    E <- eigcoW$values*(n - ng)  ##eigenvalues of between group SSPQR
+    U <- eigcoW$vectors
     Ec <- eigcoW$values
     Ec2 <- Ec
-    
-    #if (min(E) < tolinv)
-    #    cat(paste("singular Covariance matrix: General inverse is used. Threshold for zero eigenvalue is", tolinv, "\n"))
-    for (i in 1:length(eigW$values)) {
-        if (Ec[i] < tolinv) {
-            E[i] <- Ec[i] <- Ec2[i] <- 0
-        } else {
-            E[i] <- sqrt(1/E[i])
-            Ec[i] <- sqrt(1/Ec[i])
-            Ec2[i] <- (1/Ec2[i])
-        }
+    geninv <- FALSE
+    if (min(Ec) < tolinv) {
+        #cat(paste("singular Covariance matrix: General inverse is used. Threshold for zero eigenvalue is", tolinv, "\n"))
+        geninv <- TRUE
     }
-    invcW <- diag(Ec)
-    irE <- diag(E)
-    ZtZ <- irE %*% t(U) %*% t(X) %*% X %*% U %*% irE
-    eigZ <- eigen(ZtZ,symmetric=TRUE)
+    abovetol <- which(Ec >= tolinv)
+    E[abovetol] <- sqrt(1/E[abovetol])
+    Ec[abovetol] <- sqrt(1/Ec[abovetol])
+    Ec2[abovetol] <- 1/Ec2[abovetol]
+    if (geninv)
+        Ec[-abovetol] <- E[-abovetol] <- Ec2[-abovetol] <- 0
+
     useEig <- min((ng-1), l)
-    A <- Re(eigZ$vectors[, 1:useEig])
-    CV <- U %*% invcW %*% A
+    ZtZ <- (E * t(X %*% U))
+    eigZ <- svd(ZtZ,nv=0,nu=useEig)
+    eigZ$d <- eigZ$d^2
+    A <- Re(eigZ$u)
+    CV <- U %*% (Ec * A)
     di <- dim(CV)[2]
     
     for (i in 1:di) {
