@@ -1,11 +1,16 @@
 #' calculate an affine transformation matrix
 #'
 #' calculate an affine transformation matrix
-#' @param x fix landmarks
-#' @param y moving landmarks
+#' @param x fix landmarks. Can be a k x m matrix or mesh3d.
+#' @param y moving landmarks. Can be a k x m matrix or mesh3d.
 #' @param type set type of affine transformation: options are  "rigid", "similarity" (rigid + scale) and "affine",
 #' @param reflection logical: if TRUE "rigid" and "similarity" allow reflections.
 #' @param lambda numeric: regularisation parameter of the TPS.
+#' @param weights vector of length k, containing weights for each landmark (only used in type="rigid" or "similarity").
+#' @param centerweight logical: if weights are defined and centerweigths=TRUE,
+#' the matrix will be centered according to these weights instead of the
+#' barycenter.
+#' @param threads number of threads to use in TPS interpolation.
 #' @details
 #' \code{x} and \code{y} can also be a pair of meshes with corresponding vertices.
 #' @return returns a 4x4 (3x3 in 2D case)  transformation matrix or an object of class "tpsCoeff" in case of type="tps".
@@ -14,8 +19,9 @@
 #' data(boneData)
 #' trafo <- computeTransform(boneLM[,,1],boneLM[,,2])
 #' transLM <- applyTransform(boneLM[,,2],trafo)
+#' @seealso \code{\link{rotonto}, link{rotmesh.onto}, \link{tps3d}}
 #' @export
-computeTransform <- function(x,y,type=c("rigid","similarity","affine","tps"),reflection=FALSE,lambda=1e-8) {
+computeTransform <- function(x,y,type=c("rigid","similarity","affine","tps"),reflection=FALSE,lambda=1e-8, weights=NULL,centerweight=FALSE,threads=1) {
     if (inherits(x,"mesh3d"))
         x <- vert2points(x)
      if (inherits(y,"mesh3d"))
@@ -36,7 +42,7 @@ computeTransform <- function(x,y,type=c("rigid","similarity","affine","tps"),ref
         scale <- TRUE
         if (type == "r")
             scale <- FALSE
-        trafo <- getTrafo4x4(rotonto(x,y,scale = scale,reflection=reflection))
+        trafo <- getTrafo4x4(rotonto(x,y,scale = scale,reflection=reflection,weights=weights,centerweight=centerweight))
     } else if (type=="a"){
         k <- nrow(x)
         m <- ncol(x)
@@ -53,10 +59,9 @@ computeTransform <- function(x,y,type=c("rigid","similarity","affine","tps"),ref
         trafo[m+1,m+1] <- 1
     } else if (type == "t") {
         m <- ncol(y)
-        Lall <- CreateL(y,lambda=lambda, output="Linv")
-        Linv <- Lall$Linv
+        L <- CreateL(y,lambda=lambda, output="L",threads=threads)$L
         m2 <- rbind(x,matrix(0,m+1,m))
-        coeff <- as.matrix(Linv%*%m2)
+        coeff <- as.matrix(base::solve(L,m2))
         trafo <- list(refmat=y,tarmat=x,coeff=coeff,lambda=lambda)
         class(trafo) <- "tpsCoeff"
     } else {
