@@ -11,9 +11,6 @@ SEXP asymPerm(SEXP asymr, SEXP groupsr, SEXP roundr) {
     mat armaAsym =as<mat>(asymr);
     arma::ivec armaGroups = Rcpp::as<arma::ivec>(groupsr);
     int rounds = Rcpp::as<int>(roundr);
-    int n = armaAsym.n_rows;
-    int m = armaAsym.n_cols;
-    
     ivec permuvec = armaGroups;
     std::vector<double> diff,angle;
     for (int i=0; i <= rounds; ++i) {
@@ -47,8 +44,6 @@ RcppExport SEXP asymPermute(SEXP data_, SEXP groups_, SEXP rounds_) {
     mat armaData = as<mat>(data_);
     arma::ivec armaGroups = Rcpp::as<arma::ivec>(groups_);
     int rounds = Rcpp::as<int>(rounds_);
-    int n = armaData.n_rows;
-    int m = armaData.n_cols;
     ivec permuvec = armaGroups;
     int maxlev = armaGroups.max();
     int alldist=0;
